// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#include "ConfigurationSlotValue.h"

#ifndef TextConfigurationEditor_h_included
#include "TextConfigurationEditor.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef BadSlotValue_h_included
#include "BadSlotValue.h"
#endif

struct ConfigurationSlotValue::ConfigurationSlotValueData {
  SP<Configuration> m_configuration;
  FString m_name;
  ConfigurationSlotValueData (const ConfigurationSlotValueData &csvd)
    : m_configuration (csvd.m_configuration->copy()),
      m_name (csvd.m_name)
  {}
  ConfigurationSlotValueData () {}
};

ConfigurationSlotValue::ConfigurationSlotValue (const ConfigurationSlotValue &r)
  : SlotValue (r),
    m_data (NEW (ConfigurationSlotValueData (*(r.m_data))))
{}

ConfigurationSlotValue::ConfigurationSlotValue (const Configuration &conf,
                                                const FString &name)
  : SlotValue (name),
    m_data (NEW (ConfigurationSlotValueData ()))
{
  m_data->m_configuration = conf.copy ();
  m_data->m_name = name;
}

ConfigurationSlotValue::~ConfigurationSlotValue () {
  assert (m_data);
  delete m_data;
  m_data = 0;
}

const Configuration *ConfigurationSlotValue::getConfiguration () const {
  assert (m_data->m_configuration);
  return m_data->m_configuration;
}

Configuration *ConfigurationSlotValue::getConfiguration () {
  assert (m_data->m_configuration);
  return m_data->m_configuration;
}

void ConfigurationSlotValue::setConfiguration (SP<Configuration> c) {
  assert (c->hasTheSameTypeAs (&*m_data->m_configuration));
  m_data->m_configuration = c;
}

bool ConfigurationSlotValue::hasTheSameTypeAs (const SlotValue *s) const {
  SP<const ConfigurationSlotValue> theCSV =
    dynamic_cast <const ConfigurationSlotValue *> (s);
  return (theCSV &&
          getConfiguration()->hasTheSameTypeAs (theCSV->getConfiguration()));
}

FString ConfigurationSlotValue::unParse () const {
  return getConfiguration()->unParse ();
}

void ConfigurationSlotValue::editSlotValue () {
  TextConfigurationEditor::edit (getConfiguration ());
}

SP<SlotValue> ConfigurationSlotValue::copy () const {
  return NEW (ConfigurationSlotValue (*this));
}

SP<ConfigurationSlotValue> ConfigurationSlotValue::getSlotValue
(Configuration * a_c, const FString &name)
{
  SP<Configuration> c = a_c;
  SP<SlotValue> slot = c->getSlot (name);
  ConfigurationSlotValue *islot =
    dynamic_cast <ConfigurationSlotValue *> (&*slot);
  assert (islot);
  return islot;
}

SP<const ConfigurationSlotValue> ConfigurationSlotValue::getSlotValue
(const Configuration *a_c, const FString &name)
{
  SP<const Configuration> c = a_c;
  SP<const SlotValue> slot = c->getSlot (name);
  const ConfigurationSlotValue *islot =
    dynamic_cast <const ConfigurationSlotValue *> (&*slot);
  assert (islot);
  return islot;
}

void ConfigurationSlotValue::setSlotValue (SP<Configuration> c,
                                           const FString &name,
                                           SP<ConfigurationSlotValue> rsv)
{
  SP<BadSlotValue> b = c->setSlot (name, &*rsv);
  assert (!b);
}
