// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef Configurable_h_included
#include "Configurable.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef Action_h_included
#include "Action.h"
#endif

#ifndef TransformConfiguration_h_included
#include "TransformConfiguration.h"
#endif

#ifndef SceneGraph_h_included
#include "SceneGraph.h"
#endif

#ifndef CursorBall_h_included
#include "CursorBall.h"
#endif

#ifndef OL_h_included
#include "OL.h"
#endif

#ifndef TopLevel_h_included
#include "TopLevel.h"
#endif

#ifndef CursorBallManager_h_included
#include "CursorBallManager.h"
#endif

#ifndef Matrix4_h_included
#include "Matrix4.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef Canvas_h_included
#include "Canvas.h"
#endif

namespace {
  class TransformCursorBallRHScreen
    : public Factory
  {
  public:
    TransformCursorBallRHScreen ()
      : Factory ("TransformCursorBallRHScreen")
    {}
    SP<Configuration> defaultConfiguration () const {
      return NEW (TransformConfiguration ());
    }
    FString getPrettyName (const Configuration *c) const {
      (void) c;
      return "move the Cursor Ball";
    }
    SP<Configurable> makeIt (SP<Configuration> c) const {
      const SP<TransformConfiguration> tc =
	dynamic_cast <TransformConfiguration *> (&*c);
      assert (tc);
      const SP<TopLevel> top = tc->getTopLevel();
      SP<SceneGraph> sg = top->getSceneGraph ();
      // Don't copy here, otherwise moveTo will not update the global cursor
      // ball when it is configured to have a different size.
      OL<CursorBall> &cbl = CursorBallManager::getCursorBall (sg);
      // The center is probably the location of the CursorBall, but we don't
      // assume that.  Note that the CursorBall's position and rotation are in
      // world coordinates, but the tc means screen coordinates.
      SP<Canvas> canvas = top->getCanvas ();
      cbl->moveTo (cbl,
		   canvas->transformRHScreenToWorld
		   (tc->getMatrix () *
		    canvas->transformWorldToRHScreen (cbl->getPosition())),
		   canvas->transformRHScreenToWorld
		   (tc->getRotation () *
		    canvas->transformWorldToRHScreen (cbl->getRotation())));
      return NEW (Action ());
    }
  };
  static const bool useless =
  (FactoryTable::store ("TransformAction",
			NEW (TransformCursorBallRHScreen ())),
   true);
}
