// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef InputDevicesFactory_h_included
#include "InputDevicesFactory.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef InputDevices_h_included
#include "InputDevices.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef stdlib_h_included
#include <stdlib.h>
#define stdlib_h_included
#endif

#ifndef iostream_h_included
#include <iostream>
#define iostream_h_included
#endif

#ifndef ConstantRecursiveSlotValue_h_included
#include "ConstantRecursiveSlotValue.h"
#endif

#ifndef Configurable_h_included
#include "Configurable.h"
#endif

#ifndef InputDevice_h_included
#include "InputDevice.h"
#endif

InputDevicesFactory::InputDevicesFactory (const FString &name)
  : Factory (name)
{}

void InputDevicesFactory::openDevices
(const Dynavec <FString> &prefixes,
 Dynavec <SP <InputDevice> > &inputDevices)
{
  inputDevices.extendTo (0);
  Dynavec <SP<Factory> > factories =
    FactoryTable::load ("InputDevicesFactory");
  Dynavec <SP<InputDevicesFactory> > idfs;
  for (int i = 0; i < factories.size(); i++) {
    SP<InputDevicesFactory> idf =
      dynamic_cast <InputDevicesFactory *> (&*factories[i]);
    assert (idf);
    for (int j = 0; j < prefixes.size(); j++) {
      if (idf->getName().substring (0, prefixes[j].size()) == prefixes[j]) {
	idfs.push (idf);
	break;
      }
    }
  }
  SP<InputDevice> result;
  while (idfs.size ()) {
    bool progress = false;
    for (int i = 0; i < idfs.size(); ) {
      SP<InputDevicesFactory> idf = idfs[i];
      bool goodEnough = true;
      for (int j = 0; j < idfs.size(); j++) {
	Dynavec <SP <InputDevicesFactory> > worse =
	  idfs[j]->inferiorDeviceFactories ();
	for (int k = 0; k < worse.size(); k++) {
	  if (worse [k] == idf) {
	    goodEnough = false;
	  }
	}
      }
      if (goodEnough) {
	progress = true;
	// Take it out of the list.  Note that we don't increment i in this
	// case.
	idfs [i] = idfs [idfs.size() - 1];
	idfs.pop ();
	// Run it.
	// FIXME Give the user a chance to configure InputDevices
	// We aren't giving the user any chance to have anything but the
	// default configurations here, which is as good as no configuration at
	// all.  But all this happens before we start doing serious I/O with
	// the user anyway, so maybe it doesn't matter that we aren't
	// giving the user a chance to interact here.
	// If we had a way to save the configuration from one run to the next,
	// then we could restore the configuration for the input device.
	SP<Configurable> c = idf->makeIt ();
	SP<InputDevices> ids = dynamic_cast <InputDevices *> (&*c);
	assert (ids);
	Dynavec <SP <InputDevice> > did;
	ids->findDevices(did);
	for (int j = 0; j < did.size(); j++) {
	  inputDevices.push (did [j]);
	}
      } else {
	i++;
      }
    }
    if (!progress) {
      std::cerr << "The following InputDevicesFactory's all want to run before "
	   << std::endl
	   << "each other, so there is no good order to run them:" << std::endl;
      for (int i = 0; i < idfs.size(); i++) {
	std::cerr << idfs[i]->getName() << std::endl;
      }
      abort ();
    }
  }
}
