// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//
#ifndef ChordMouseDragConfiguration_h_included
#include "ChordMouseDragConfiguration.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef VectorSlotValue_h_included
#include "VectorSlotValue.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef RecursiveSlotValue_h_included
#include "RecursiveSlotValue.h"
#endif

#ifndef MouseDragBindingConfiguration_h_included
#include "MouseDragBindingConfiguration.h"
#endif

#ifndef ConfigurationSlotValue_h_included
#include "ConfigurationSlotValue.h"
#endif

#ifndef VecUtil_h_included
#include "VecUtil.h"
#endif

namespace {
  FString bindingsName () {
    return "Bindings";
  }
  FString defaultName () {
    return "Default Action";
  }
}

ChordMouseDragConfiguration::ChordMouseDragConfiguration () {
  SP<Configuration> mdbc = NEW (MouseDragBindingConfiguration ());
  initializeSlot (bindingsName (),
                  NEW (VectorSlotValue
                       (NEW (ConfigurationSlotValue (*mdbc,
                                                     "mouse drag binding")),
                        Dynavec <SP<SlotValue> > ())));
  initializeSlot (defaultName (),
                  NEW (RecursiveSlotValue ("Action",
                                           "DoNothing")));
}

ChordMouseDragConfiguration::ChordMouseDragConfiguration
(const ChordMouseDragConfiguration &tc)
  : ActionConfiguration (tc)
{}

SP<Configuration> ChordMouseDragConfiguration::copy () const {
  return NEW (ChordMouseDragConfiguration (*this));
}

bool ChordMouseDragConfiguration::hasTheSameTypeAs (const Configuration *c) const {
  return 0 != dynamic_cast <const ChordMouseDragConfiguration *> (c);
}

SP<const VectorSlotValue> ChordMouseDragConfiguration::getBindings () const {
  return VectorSlotValue::getSlotValue (this, bindingsName ());
}

SP<VectorSlotValue> ChordMouseDragConfiguration::getBindings () {
  return VectorSlotValue::getSlotValue (this, bindingsName ());
}

SP<RecursiveSlotValue> ChordMouseDragConfiguration::getDefaultAction () {
  return RecursiveSlotValue::getSlotValue (this, defaultName ());
}

SP<const RecursiveSlotValue> ChordMouseDragConfiguration::getDefaultAction () const {
  return RecursiveSlotValue::getSlotValue (this, defaultName ());
}

void ChordMouseDragConfiguration::setDefaultAction (RecursiveSlotValue *rsv) {
  RecursiveSlotValue::setSlotValue (this, defaultName (), rsv);
}

bool ChordMouseDragConfiguration::isBound (const Event::Button b) const {
  const SP<const VectorSlotValue> vsv = getBindings ();
  for (int i = 0; i < vsv->size(); i++) {
    const SP<const ConfigurationSlotValue> csv =
      static_cast <const ConfigurationSlotValue *> (&*((*vsv)[i]));
    const SP<const MouseDragBindingConfiguration> mdbc =
      static_cast <const MouseDragBindingConfiguration *>
      (csv->getConfiguration ());
    if (VecUtil::find (b, mdbc->getShifts())) {
      return true;
    }
  }
  return false;
}
