#ifndef ChordMouseDragConfiguration_h_included
#define ChordMouseDragConfiguration_h_included

// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//
#ifndef ActionConfiguration_h_included
#include "ActionConfiguration.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef Event_h_included
#include "Event.h"
#endif

class VectorSlotValue;
class RecursiveSlotValue;
class ChordMouseDragConfiguration : public ActionConfiguration {
  // This class allows configuring a PrefixMouseDrag, which allows things like
  // typing an R and then dragging the middle mouse button to add a vector to
  // the velocity of the selection.  The connection between R and this Action
  // comes from the keyboard binding.  The adding of the velocity comes from a
  // MouseDrag that the PrefixMouseDrag is configured to invoke.
  // This class configures the class
  // that provides the glue that understands what how the mouse dragged.
public:
  ChordMouseDragConfiguration ();
  ChordMouseDragConfiguration (const ChordMouseDragConfiguration &tc);
  SP<Configuration> copy () const;
  bool hasTheSameTypeAs (const Configuration *c) const;

  // getBindings returns a VectorSlotValue where each one is a
  // MouseDragBindingConfiguration.
  SP<const VectorSlotValue> getBindings () const;
  SP<VectorSlotValue> getBindings ();
  // getDefaultAction is a configured Action to take if none of the bindings
  // apply.
  SP<RecursiveSlotValue> getDefaultAction ();
  SP<const RecursiveSlotValue> getDefaultAction () const;
  void setDefaultAction (RecursiveSlotValue *rsv);
  // Whether the given button is bound to some non-default value in this
  // ChordMouseDragConfiguration.  This ignores shift keys. 
  bool isBound (Event::Button b) const;
};

#endif
