// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef MouseDragConfiguration_h_included
#define MouseDragConfiguration_h_included

#ifndef Point_h_included
#include "Point.h"
#endif

#ifndef ActionConfiguration_h_included
#include "ActionConfiguration.h"
#endif

#ifndef Action_h_included
#include "Action.h"
#endif

class TopLevel;
class RecursiveSlotValue;
class KeyboardMouseEvent;
class MouseDragConfiguration
// A Configuration for a piece of code that knows what to do with a mouse drag.
  : public ActionConfiguration
{
private:
  class MouseDragConfigurationData;
  MouseDragConfigurationData *m_data;
public:
  MouseDragConfiguration ();
  MouseDragConfiguration (const MouseDragConfiguration &mdc);
  virtual ~MouseDragConfiguration ();
  // This is useless to configure interactively, so all of the slots are simple
  // fields in the structure.

  // m_commitment is FIRST if this is the initial button press of the mouse; in
  // this case the end coordinates will be equal to the start coordinates.  It
  // will be TENTATIVE if we're doing mouse motion after he pressed the
  // mouse button down.  It's COMMIT if we're dealing with the release of a
  // mouse button.

  // The caller will guarantee that m_commitment will be FIRST the first time,
  // then it will be TENTATIVE zero or more times, then it will be COMMIT the
  // last time.

  // (I thought of having ABORT as an option and decided against it.  It would
  // have meant that the user chose to abort the command and wants things to go
  // back to the way they were before.  Some issues with that:
  // 1. It's redundant with having an undo capability.
  // 2. Existing mouse dragging commands don't have the capability so I don't
  // know that it's needed.
  // 3. I could get the same thing for most reasonable mouse dragging commands
  // by doing a commit with the end coordinates equal to the start
  // coordinates.
  // So I'm leaving it out.)
  typedef enum {FIRST, TENTATIVE, COMMIT} CommitmentLevel;
  void setCommitment (CommitmentLevel cl);
  CommitmentLevel getCommitment () const;
  // The initial coordinates of the mouse.  When you call setStart it also sets
  // the end and prev to the start value.
  void setStart (const Point &p);
  Point getStart () const;
  // The final coordinates of the mouse, or the current coordinates if m_final
  // is false.  When the mouse initially goes down, getStart and getEnd and
  // getPrev all return the same value.
  void setEnd (const Point &p);
  Point getEnd () const;
  // The previous mouse coordinates, when the mouse is being dragged.  getPrev
  // should return the same value that getEnd did last time.  We don't need
  // setPrev, since setEnd will do the work.
  Point getPrev () const;
  SP<Configuration> copy () const;
  bool hasTheSameTypeAs (const Configuration *c) const;
  // startDrag, mouseMoved, and endDrag all take an RSV which had better have a
  // MouseDragConfiguration as its Configuration. 
  static SP<Action> startDrag (const KeyboardMouseEvent *kme,
                         RecursiveSlotValue *rsv,
                         TopLevel *top);
  static SP<Action> mouseMoved (const KeyboardMouseEvent *kme,
                          RecursiveSlotValue *rsv);
  static SP<Action> endDrag (const KeyboardMouseEvent *kme, RecursiveSlotValue *rsv);
};
#endif
