// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef Ball_h_included
#define Ball_h_included

#ifndef FFloat_h_included
#include "FFloat.h"
#endif

#ifndef PhysicsObject_h_included
#include "PhysicsObject.h"
#endif

#ifndef Vec3_h_included
#include "Vec3.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef Color_h_included
#include "Color.h"
#endif

class BoundingBox;
class BallConfiguration;
class Color;

// A ball.
// The visible part of a spherical physics object.
// We assume the first 3 state variables are the center of the visible image.
class Ball : public PhysicsObject {
  const FFloat m_visibleRadius;
  const Color m_material;
public:
  Ball (SP<BallConfiguration> b);
  
  inline FFloat visibleRadius () const {
    return m_visibleRadius;
  }

  // A subclass might want to redefine stateSize to be larger.
  virtual int stateSize () const;
  void translate (FFloat *state, const Vec3 &v) const;
  // Next two do nothing.
  void addVelocity (FFloat *state, const Vec3 &v) const;
  void rotate (FFloat *state, const Quaternion &q) const;
  Vec3 boundingSphereCenter (const FFloat *state) const;
  inline static Vec3 center (const FFloat *state) {
    return Vec3 (state);
  }
  static void setCenter (FFloat *state, const Vec3 &v);
  ~Ball ();
  static const char *const staticClassId;
  Color getMaterial () const;

};

#endif
