// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//
#ifndef DDPOConfiguration_h_included
#include "DDPOConfiguration.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef MenuSlotValue_h_included
#include "MenuSlotValue.h"
#endif

#ifndef MenuSlotValueChoices_h_included
#include "MenuSlotValueChoices.h"
#endif

namespace {
  FString drawingStyleName () {
    return "Drawing style";
  }
}

SP<MenuSlotValue> DDPOConfiguration::getMenuSlot () {
  return MenuSlotValue::getSlotValue (this, drawingStyleName ());
}

SP<const MenuSlotValue> DDPOConfiguration::getMenuSlot () const {
  return MenuSlotValue::getSlotValue (this, drawingStyleName ());
}

DDPOConfiguration::DDPOConfiguration (MenuSlotValueChoices *choices) {
  initializeSlot (drawingStyleName (), NEW (MenuSlotValue (choices)));
}

const FString DDPOConfiguration::getDrawerName () const {
  return getMenuSlot () -> getShortStringChoice ();
}

void DDPOConfiguration::setDrawerName (const FString &name) {
  return getMenuSlot () -> setShortStringChoice (name);
}

void DDPOConfiguration::setDrawerPos (int pos) {
  return getMenuSlot () -> setIntChoice (pos);
}

int DDPOConfiguration::generation () const {
  return getMenuSlot () -> getChoices () -> generation ();
}

int DDPOConfiguration::numAlternatives () const {
  return getMenuSlot () -> getChoices () -> size ();
}

int DDPOConfiguration::currentDrawerPos () const {
  return getMenuSlot () -> getIntChoice ();
}

SP<Configuration> DDPOConfiguration::copy () const {
  return NEW (DDPOConfiguration (*this));
}

bool DDPOConfiguration::hasTheSameTypeAs (const Configuration *c) const {
  return 0 != dynamic_cast <const DDPOConfiguration *> (c);
}
