// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef GenericObjectDrawer_h_included
#include "GenericObjectDrawer.h"
#endif

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef ObjectDrawerConfiguration_h_included
#include "ObjectDrawerConfiguration.h"
#endif

#ifndef ConstantRecursiveSlotValue_h_included
#include "ConstantRecursiveSlotValue.h"
#endif

#ifndef OneObjectDrawerConfiguration_h_included
#include "OneObjectDrawerConfiguration.h"
#endif

#ifndef Configurable_h_included
#include "Configurable.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

namespace {
  class ObjectDrawerFactory
    : public Factory
  {
    FString m_windowSystem;
  public:
    ObjectDrawerFactory (const FString &windowSystem)
      : Factory (windowSystem+"ObjectDrawer"),
	m_windowSystem (windowSystem)
    {}
    SP<Configuration> defaultConfiguration () const {
      return NEW (ObjectDrawerConfiguration (m_windowSystem));
    }
    SP<Configurable> makeIt (SP<Configuration> c) const {
      SP<ObjectDrawerConfiguration> odconf =
	dynamic_cast <ObjectDrawerConfiguration *> (&*c);
      assert (odconf);
      SP<ConstantRecursiveSlotValue> crsv =
	dynamic_cast <ConstantRecursiveSlotValue *>
	(&*odconf->getSlot (odconf->getClassName()));
      assert (crsv);
      SP<OneObjectDrawerConfiguration> oodconf =
	dynamic_cast <OneObjectDrawerConfiguration *>
	(crsv->getConfiguration ());
      assert (oodconf);
      oodconf->setPhysicsObjectConfiguration
	(odconf->getPhysicsObjectConfiguration ());
      return crsv->makeIt ();
    }
  };
};

// FIXME Can't have a PhysicsObject called "Object"
// Because OGLObjectDrawer is the factory that makes OGLBallDrawer,
// OGLDiskDrawer, etc.
// FIXME Names of pieces run together in factory, hence ambiguity.
//
// For example, window system OGLB, object all vs window system OGL, object
// Ball.
//
// A fix would be to have an underscore in the name and to have a convention
// (and an assertion) that window systems don't have underscores in their
// names.
void GenericObjectDrawer::registerWindowSystem (const FString &windowSystem) {
  FactoryTable::store ("ObjectDrawer",
		       NEW (ObjectDrawerFactory (windowSystem)));
}
