// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#include "ObjectDrawerConnection.h"

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef TheObjectDrawerFactory_h_included
#include "TheObjectDrawerFactory.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef iostream_h_included
#include <iostream>
#define iostream_h_included
#endif

struct ObjectDrawerConnection::ObjectDrawerConnectionData {
  // The name of the drawer class, "Ball" or "Disk" or whatever.
  // Doesn't include characters for the window system.
  FString m_objectDrawerClass;
  // When the global object drawer factory is changed, the singleton
  // will have to come by and call setObjectDrawer on this class.
  // The position is an integer that will be the position of this
  // ObjectDrawerConnection in its vector of odc's.
  int m_position;
  SP<Configuration> m_conf;
  SP<ObjectDrawer> m_objectDrawer;
  SP<TheObjectDrawerFactory> m_todf;
};

ObjectDrawerConnection::ObjectDrawerConnection (const FString &odc,
						SP<Configuration> conf)
  : m_data (NEW (ObjectDrawerConnectionData ()))
{
  m_data->m_objectDrawerClass = odc;
  // Copy the configuration so it wont change out from under us.
  m_data->m_conf = conf->copy();
  // Don't want smart pointer nonsense in registerConnection to free this.
  ref ();
  TheObjectDrawerFactory::registerConnection (this);
  deref ();
}

ObjectDrawerConnection::~ObjectDrawerConnection () {
  // ref/deref is to make sure deRegisterConnection smart pointer
  // manipulation doesn't free the thing we're in the middle of
  // destructing.
  ref ();
  m_data->m_todf->deRegisterConnection (this);
  deref ();
  assert (m_data);
  delete m_data;
  m_data = 0;
}

FString ObjectDrawerConnection::getObjectDrawingClass () const {
  return m_data->m_objectDrawerClass;
};

void ObjectDrawerConnection::setObjectDrawer
(SP<ObjectDrawer> od, SP<TheObjectDrawerFactory> todf) {
  m_data->m_objectDrawer = od;
  m_data->m_todf = todf;
}

void ObjectDrawerConnection::setObjectDrawerPosition (int position) {
  m_data->m_position = position;
}

int ObjectDrawerConnection::getObjectDrawerPosition () const {
  return m_data->m_position;
}

SP<Configuration> ObjectDrawerConnection::getConfiguration () {
  return m_data->m_conf;
}

const ObjectDrawer *ObjectDrawerConnection::getObjectDrawer () const {
  assert (m_data->m_objectDrawer);
  return m_data->m_objectDrawer;
}

ObjectDrawer *ObjectDrawerConnection::getObjectDrawer () {
  assert (m_data->m_objectDrawer);
  return m_data->m_objectDrawer;
}
