// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef ActionConfiguration_h_included
#include "ActionConfiguration.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef Configurable_h_included
#include "Configurable.h"
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef ObjectLink_h_included
#include "ObjectLink.h"
#endif

#ifndef FloatUtil_h_included
#include "FloatUtil.h"
#endif

#ifndef iostream_h_included
#include <iostream>
#define iostream_h_included
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef TopLevel_h_included
#include "TopLevel.h"
#endif

#ifndef SelectionManager_h_included
#include "SelectionManager.h"
#endif

#ifndef Action_h_included
#include "Action.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

namespace {
  class GrowSelectionNearby
    : public Factory
  {
  public:
    GrowSelectionNearby ()
      : Factory ("GrowSelectionNearby")
    {}
    SP<Configuration> defaultConfiguration () const {
      return NEW (ActionConfiguration ());
    }
    SP<Configurable> makeIt (const SP<Configuration> c) const {
      SP<ActionConfiguration> conf =
	dynamic_cast <ActionConfiguration *> (&*c);
      assert (conf);
      SP<TopLevel> top = conf->getTopLevel ();
      Dynavec <int> selection;
      SelectionManager::getSelection (selection);
      // FIXME We could do faster by using bins.
      // See DESIGN.NEAREST.  Or, just use any of the Storage classes and have
      // an a-priori maximum distance of a few angstroms.  But release the slow
      // implementation for now.
      SP<SceneGraph> sg = top->getSceneGraph ();
      // The set of objects selected.
      // somethingIn will eventually hold the index of an object that is
      // selected.
      ObjectLink somethingIn;
      Dynavec <bool> selectionSet;
      selectionSet.extendTo (sg->maxIndex(), false);
      for (int i = 0; i < selection.size(); i++) {
	const int s = selection [i];
	assert (-1 != s);
	selectionSet [s] = true;
	if (!somethingIn.isValid ()) {
	  somethingIn = ObjectLink (sg, s);
	  assert (somethingIn.isValid ());
	}
      }
      SP<Action> result = NEW (Action ());
      if (!somethingIn.isValid ()) {
	result->setProblem ("You must select something before growing "
			    "the selection.");
      }
      // somethingOut will eventually hold the index of an object that is not
      // selected.
      ObjectLink somethingOut;
      // How far the nearest thing in the scene graph is from something in the
      // set.
      // Compiler thinks nearDistance may be used uninitialized unless I
      // initialize it here.  It's wrong.  No big deal.
      FFloat nearestDistance = 0;
      if (!result->isProblem ()) {
	for (int i = 0; i < sg->maxIndex(); i++) {
	  if (!selectionSet [i] && sg->object (i)->isSelectable()) {
	    ObjectLink po (sg, i);
	    if (po.isValid () && po.isVisible ()) {
	      // Now we know po points to something not selected.
	      if (!somethingOut.isValid ()) {
		somethingOut = po;
		assert (somethingOut.isValid ());
		// Initialize nearestDistance to any distance between something
		// in the set and something out of the set.
		nearestDistance =
		  (somethingOut.boundingSphereCenter () -
		   somethingIn.boundingSphereCenter()).lengthSquared ();
	      }
	      for (int j = 0; j < selection.size(); j++) {
		ObjectLink thisSel (sg, selection[j]);
		if (thisSel.isVisible ()) {
		  nearestDistance =
		    FloatUtil::min (nearestDistance,
				    (po.boundingSphereCenter() -
				     thisSel.boundingSphereCenter())
				    .lengthSquared());
		}
	      }
	    }
	  }
	}
	if (!somethingOut.isValid ()) {
	  result->setProblem ("The entire scene is selected, so the selection "
			      "cannot grow any more.");
	}
      }
      if (!result->isProblem ()) {
	// Make sure nearestDistance increases more than roundoff error each
	// time.
	// FIXME The 1.01 should be a configuration constant.
	nearestDistance *= 1.01;
	message (FString ("Adding everything closer than ") +
		 FloatUtil::sqrt (nearestDistance));
	for (;;) {
	  bool grown = false;
	  for (int i = 0; i < sg->maxIndex(); i++) {
	    if (!selectionSet [i]) {
	      ObjectLink po (sg, i);
	      if (po.isValid () && SelectionManager::isSelectable (po)) {
		for (int j = 0; j < selection.size (); j++) {
		  ObjectLink thisSel (sg, selection [j]);
		  if ((po.boundingSphereCenter()-
		       thisSel.boundingSphereCenter()).lengthSquared()
		      <= nearestDistance) {
		    selection.push (i);
		    selectionSet [i] = true;
		    grown = true;
		    break;
		  }
		}
	      }
	    }
	  }
	  if (!grown) break;
	}
	SelectionManager::setSelection (sg, selection);
	result->setMessage (FString (selection.size())+ " objects selected");
      }
      return &*result;
    }
  };
  const bool useless =
  (FactoryTable::store ("Action", NEW (GrowSelectionNearby ())),
   true);
}
