// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef Configurable_h_included
#include "Configurable.h"
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef ActionConfiguration_h_included
#include "ActionConfiguration.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef TopLevel_h_included
#include "TopLevel.h"
#endif

#ifndef SelectionManager_h_included
#include "SelectionManager.h"
#endif

#ifndef SceneGraph_h_included
#include "SceneGraph.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef Action_h_included
#include "Action.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef LinkManager_h_included
#include "LinkManager.h"
#endif

#ifndef PhysicsObject_h_included
#include "PhysicsObject.h"
#endif

#ifndef Vec3_h_included
#include "Vec3.h"
#endif

namespace {
  class ToggleConnectedSelection
    : public Factory
  {
  public:
    ToggleConnectedSelection ()
      : Factory ("ToggleConnectedSelection")
    {}
    SP<Configuration> defaultConfiguration () const {
      return NEW (ActionConfiguration ());
    }
    SP<Configurable> makeIt (const SP<Configuration> c) const {
      SP<ActionConfiguration> conf =
	dynamic_cast <ActionConfiguration *> (&*c);
      assert (conf);
      SP<TopLevel> top = conf->getTopLevel ();
      // The index of the selected object.
      int index;
      SP<Action> result = NEW (Action ());
      SP<SceneGraph> sg = top->getSceneGraph ();
      {
	// The point in space where the user clicked on the selected object.
	Vec3 intersectionPoint;
	if (!top->findUnderMouse (index, intersectionPoint)) {
	  result->setProblem ("There is no object under the mouse; "
			      "selection unchanged");
	}
      }
      Dynavec <bool> moleculeSet;
      const int maxIndex = sg->maxIndex ();
      if (!result->isProblem ()) {
	moleculeSet.extendTo (maxIndex, false);
	Dynavec <int> oldMolecule;
	oldMolecule.push (index);
	Dynavec <int> newMolecule;
	SP<LinkManager> lm = sg->getLinkManager ();
	for (;;) {
	  bool anyAdded = false;
	  newMolecule.extendTo (0);
	  // oldMolecule is our work queue.  newMolecule is what we'll work on
	  // next time around the loop.  If any elements in oldMolecule aren't
	  // already in the result, put them in the result and put their
	  // neighbors in newMolecule.
	  for (int i = 0; i < oldMolecule.size (); i++) {
	    const int s = oldMolecule [i];
	    // Note the call to PhysicsObject::isSelectable rather than
	    // SelectionManager::isSelectable.  We probably want to select
	    // invisible objects here, since they are likely to have the
	    // proper connections.
	    if (!moleculeSet [s] && sg->object(s)->isSelectable ()) {
	      moleculeSet [s] = true;
	      anyAdded = true;
	      int count;
	      const int *links;
	      lm->getLinks (s, count, links);
	      for (int i = 0; i < count ; i++) {
		const int thisLink = links [i];
		if (-1 != thisLink) {
		  newMolecule.push (thisLink);
		}
	      }
	    }
	  }
	  oldMolecule = newMolecule;
	  if (!anyAdded) break;
	}
	// Get the selection into selectionSet.
	Dynavec <bool> selectionSet;
	selectionSet.extendTo (maxIndex, false);
	{
	  Dynavec <int> selection;
	  SelectionManager::getSelection (selection);
	  for (int i = 0; i < selection.size (); i++) {
	    selectionSet [selection [i]] = true;
	  }
	}
	// Now selectionSet [i] is true iff object i is selected.
	bool allMoleculeIsSelected = true;
	for (int i = 0; i < maxIndex; i++) {
	  if (moleculeSet [i] && !selectionSet [i]) {
	    allMoleculeIsSelected = false;
	    break;
	  }
	}
	// Now allMoleculeIsSelected is true iff the entire molecule that they
	// clicked on is selected.
	// If all of the molecule is already selected, then deselect it.
	// Otherwise select it.  This code is redundant with
	// SelectFrustrum.cpp. 
	for (int i = 0; i < maxIndex; i++) {
	  if (moleculeSet [i]) {
	    selectionSet [i] = !allMoleculeIsSelected;
	  }
	}
	{
	  // Select selectionSet.
	  Dynavec <int> selection;
	  for (int i = 0; i < maxIndex; i++) {
	    if (selectionSet [i]) {
	      selection.push (i);
	    }
	  }
	  SelectionManager::setSelection (sg, selection);
	}
      }
      return &*result;
    }
  };
  const bool useless =
  (FactoryTable::store ("Action", NEW (ToggleConnectedSelection ())),
   true);
}
