// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef TransformConfiguration_h_included
#include "TransformConfiguration.h"
#endif

#ifndef Configurable_h_included
#include "Configurable.h"
#endif

#ifndef TopLevel_h_included
#include "TopLevel.h"
#endif

#ifndef SceneGraph_h_included
#include "SceneGraph.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef SelectionManager_h_included
#include "SelectionManager.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef Action_h_included
#include "Action.h"
#endif

#ifndef Canvas_h_included
#include "Canvas.h"
#endif

#ifndef Matrix4_h_included
#include "Matrix4.h"
#endif

#ifndef Quaternion_h_included
#include "Quaternion.h"
#endif

namespace {
  class TransformSelectionRHScreen
    : public Factory
  {
  public:
    TransformSelectionRHScreen ()
      : Factory ("TransformSelectionRHScreen")
    {}
    SP<Configuration> defaultConfiguration () const {
      return NEW (TransformConfiguration ());
    }
    FString getPrettyName (const Configuration *c) const {
      (void) c;
      return "move the selection";
    }
    SP<Configurable> makeIt (SP<Configuration> c) const {
      SP<Action> result = NEW (Action ());
      {
	Dynavec <int> selection;
	SelectionManager::getSelection (selection);
	if (0 == selection.size ()) {
	  result->setProblem ("Nothing selected");
	}
      }
      if (!result->isProblem ()) {
	const SP<TransformConfiguration> tc =
	  dynamic_cast <TransformConfiguration *> (&*c);
	assert (tc);
	SP<Canvas> canvas = tc->getTopLevel()->getCanvas ();
	SelectionManager::transformSelection
	  (canvas->transformRHScreenToWorld (tc->getMatrix ()),
	   canvas->transformRHScreenToWorld (tc->getRotation ()));
      }
      return &*result;
    }
  };
  const bool useless =
  (FactoryTable::store ("TransformAction",
			NEW (TransformSelectionRHScreen ())),
   FactoryTable::registerSubclass ("TransformAction", "Action"),
   true);
};
