#ifndef BrennerAtom_h_included
#define BrennerAtom_h_included

#ifndef PhysicsObject_h_included
#include "PhysicsObject.h"
#endif

#ifndef BoringAtom_h_included
#include "BoringAtom.h"
#endif

class AtomConfiguration;
class Factory;
class BrennerAtom
// Unlike DesignAtom, we need to export a header file for BrennerAtom because
// it will be referenced by all the C code that needs to manipulate the state.
// For instance, the inline getKtype function needs to invoke the ktype method,
// and I want the ktype method to be inline too.
  : public BoringAtom
{
  class BrennerAtomFactory;
  friend class BrennerAtomFactory;
  int thisKtype;
  BrennerAtom (AtomConfiguration *b, const Factory *f);
public:
  inline int ktype () const {
    return thisKtype;
  }
  // lessThan groups the BrennerAtom's together.  It also sorts the atoms by
  // atomic number, just in case I want to take advantage of this to speed up
  // the code someday.
  // On second thought, testing is easier if we leave the atoms in the original
  // order when possible.  So take this out.  The default method will leave
  // things in order.
  // bool lessThan (const PhysicsObject *other) const;
  static const char *const staticClassId;
  const char *classId () const;
  bool isInstanceOf (const char *classId) const;
  // Next one will return false.
  bool exempt () const;
  // physicsIntersect is inherited from BoringAtom, and always returns false.
  int doPhysics (const SceneGraphPOInterface *sg,
                 const Float *state,
                 Float *deriv,
                 int myObjectIndex) const;
  // BrennerAtom does its own neighboring, so isRelevantNeighbor always
  // returns false.
  bool isRelevantNeighbor (const PhysicsObject *o, int myIndex,
                           int neighborIndex) const;
};
  
#endif
