/* calc_dist.h - Copyright (c) 1998 Zyvex LLC.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef CALC_DIST_h_included
#define CALC_DIST_h_included

/* You'd better define the type State before including calc_dist.h. */
#ifndef API_h_included
#include "api.h"
#endif

#ifndef VECTOR_h_included
#include "bvector.h"
#endif

/* Takes a State and two atom numbers.  Puts the vector from atom j to atom i
   into rr, and returns the squared distance between them.  I tried having one
   definition for CALC_DIST with conditionals to see if rr was zero, but it
   seemed that gcc 2.95.2 didn't always optimize away the conditional test. 
*/

static inline Float CALC_DIST_NO_RR (const struct State *s, int atm_i,
                                     int atm_j)
{
  Float result;
  Float d;
#ifndef INFINITE_CUBE
  const Float *const cube = getCube (s);
#endif
  d = posx (atm_i, s) - posx (atm_j, s);
#ifndef INFINITE_CUBE
  d -= cube[0]*floor(d/cube[0] + 0.5);
#endif
  result = d * d;
  d = posy (atm_i, s) - posy (atm_j, s);
#ifndef INFINITE_CUBE
  d -= cube[1]*floor(d/cube[1] + 0.5);
#endif
  result += d * d;
  d = posz (atm_i, s) - posz (atm_j, s);
#ifndef INFINITE_CUBE
  d -= cube[2]*floor(d/cube[2] + 0.5);
#endif
  result += d * d;
  return result;
}

static inline Float CALC_DIST (const struct State *s, int atm_i, int atm_j,
                               vector *rr)
{
  Float result;
  Float d;
#ifndef INFINITE_CUBE
  const Float *const cube = getCube (s);
#endif
  d = posx (atm_i, s) - posx (atm_j, s);
#ifndef INFINITE_CUBE
  d -= cube[0]*floor(d/cube[0] + 0.5);
#endif
  result = d * d;
  rr->x = d;
  d = posy (atm_i, s) - posy (atm_j, s);
#ifndef INFINITE_CUBE
  d -= cube[1]*floor(d/cube[1] + 0.5);
#endif
  result += d * d;
  rr->y = d;
  d = posz (atm_i, s) - posz (atm_j, s);
#ifndef INFINITE_CUBE
  d -= cube[2]*floor(d/cube[2] + 0.5);
#endif
  result += d * d;
  rr->z = d;
  return result;
}

#endif
