#ifndef fungimolapi_h_included
#define fungimolapi_h_included

// This code was released into the public domain by Tim Freeman on 15 Aug 2000.
// This is the C-to-C++ version of the api, for use with Fungimol.

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef API_h_included
#include "api.h"
#endif

#ifndef VECTOR_h_included
#include "bvector.h"
#endif

#ifndef SceneGraphPOInterface_h_included
#include "SceneGraphPOInterface.h"
#endif

#ifndef FFloat_h_included
#include "FFloat.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef BrennerAtom_h_included
#include "BrennerAtom.h"
#endif

#ifndef State_h_included
#include "State.h"
#endif

static inline int firstIndex (const struct State *s) {
  /* Returns the number of the first Atom. */
  return s->getFirstIndex ();
}

static inline int pastLastIndex (const struct State * s) {
  return s->getPastLastIndex ();
}

static inline void checkIndex (int ai, const struct State *s) {
  assert (ai >= firstIndex (s));
  assert (ai < pastLastIndex (s));
}

static inline const Float *firstCoord (int ai, const struct State *s) {
  return &s->state[s->sg->objectStateIndex (ai)];
}

static inline dvector pos (int ai, const struct State * s) {
  checkIndex (ai, s);
  dvector result;
  const Float *f = firstCoord (ai, s);
  result.x = *f++;
  result.y = *f++;
  result.z = *f;
  return result;
}

static inline Double posx (int ai, const struct State * s) {
  checkIndex (ai, s);
  return firstCoord (ai, s) [0];
}

static inline Double posy (int ai, const struct State * s) {
  checkIndex (ai, s);
  return firstCoord (ai, s) [1];
}

static inline Double posz (int ai, const struct State * s) {
  checkIndex (ai, s);
  return firstCoord (ai, s) [2];
}

static inline Double prevx (int ai, const struct State *s) {
  checkIndex (ai, s);
  return s->prevAtom (ai).x;
}

static inline Double prevy (int ai, const struct State *s) {
  checkIndex (ai, s);
  return s->prevAtom (ai).y;
}

static inline Double prevz (int ai, const struct State *s) {
  checkIndex (ai, s);
  return s->prevAtom (ai).z;
}

static inline void setPrevToCurrent (int ai, struct State *s) {
  checkIndex (ai, s);
  const Float *f = firstCoord (ai, s);
  s->prevAtom (ai).x = f[0];
  s->prevAtom (ai).y = f[1];
  s->prevAtom (ai).z = f[2];
}

static inline Float rll (const struct State *s) {
  return s->rll;
}

static inline void transForcex (int i, int j, struct State * s, Float f) {
  checkIndex (i, s);
  checkIndex (j, s);
  s->force(i).x += f;
  s->force(j).x -= f;
}

static inline void transForcey (int i, int j, struct State * s, Float f) {
  checkIndex (i, s);
  checkIndex (j, s);
  s->force(i).y += f;
  s->force(j).y -= f;
}

static inline void transForcez (int i, int j, struct State * s, Float f) {
  checkIndex (i, s);
  checkIndex (j, s);
  s->force(i).z += f;
  s->force(j).z -= f;
}

static inline int getKtype (int ai, const struct State * s) {
  checkIndex (ai, s);
  const PhysicsObject *po = s->sg->object (ai);
  assert (dynamic_cast <const BrennerAtom *> (po) != 0);
  return ((const BrennerAtom *) po) -> ktype ();
}

static inline int getNoa (int ktype, const struct State *s) {
  if (ktype >= s->noa.size()) {
    return 0;
  } else {
    return s->noa[ktype];
  }
}

#ifndef INFINITE_CUBE
/* Fungimol isn't going to support periodic space any time soon, so maybe we */
/* should just discard this case. */
static inline Float *getCube (struct State * s) {
  static Float result = {0, 0, 0};
  return &result; /* FIXME */
}
#endif

static inline int squeeze (const struct State *s) {
  // All atoms are movable in Fungimol.
  return 0;
}

static inline int movable (int ai, const struct State *s) {
  return 1;
}

static inline struct NeighborState *ljNeighbors (struct State * s) {
  return s->ljNeighbors ();
}

static inline const struct NeighborState *ljNeighborsconst (const struct State * s) {
  return s->ljNeighbors ();
}

static inline struct NeighborState *caNeighbors (struct State * s) {
  return s->caNeighbors ();
}

static inline const struct NeighborState *caNeighborsconst
(const struct State * s)
{
  return s->caNeighbors ();
}
#endif
