/* kt.h - Copyright (c) 1998 Zyvex LLC.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef KT_h_included
#define KT_h_included

/* NTYPES: maximum # of different types of atoms allowed for Lennard-Jones 
 *        potential
 */
#define NTYPES 8

/* MAX_ATOMNO is the maximum atomic number, so we can size the kt table
   correctly.  I need a number that's big enough for both brennerc and
   everything Fungimol might ever do.  200 seems pretty safe.  Memory is
   cheap.
*/
#define MAX_ATOMNO 200

/* This is the maximum ktype, inclusive.  A ktype is a small internal index
   that uniquely identifies an atom type.  The kt2 array gives the mapping.
   We use elements 1 through ktmax inclusive of many arrays, including kt2,
   xmms, xm, rmaxlj, and rslj. */ 
extern int ktmax;
extern int kt[MAX_ATOMNO+1];
extern int kt2[NTYPES+1];

/* The caller had better call initkt before using any of the above
   variables.  There's another function called initkt, so call this one
   static_initkt. */
void static_initkt ();

#endif
