/* xbren2.c - Copyright (c) 1998 Zyvex LLC.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/* I found the following code surrounded by #ifdef JM_CODE ... #endif
   in bren2.c.  The code isn't presently executed, it isn't being maintained,
   and it gets in the way when I'm trying to understand stuff, so I've moved it
   here. */

Float distween_atms( BrenAtom i, BrenAtom j);
Float is_bonded( BrenAtom x, BrenAtom y);
Float dihedral(BrenAtom i, BrenAtom j, Float ijConj);
Float energy( BrenAtom *pAtm );
Float bond_order( BrenAtom i, BrenAtom j );
Float pSigmaPi( BrenAtom i, BrenAtom j );
static Float theta( BrenAtom i, BrenAtom j, BrenAtom k );
void plot_TOR(void);
void plot_RADIC(void);
void plot_BCUINT(void);
void plot_G(void);
Float attractive_e( BrenAtom x, BrenAtom y );
Float repulsive_e( BrenAtom x, BrenAtom y );

void plot_G()
{
	FILE *outf;
	Float theta;
	BrenAtom atom;

	outf = fopen("c:\\temp.ent", "w");
	fprintf(outf, "carbon\n");
	atom.type = CARBON; 
	for (theta=0; theta<6.28; theta+=.1) {
		atom.Totbonds=0;
		while (atom.Totbonds<5) {
			fprintf(outf, "G(%.1f,%.1f)=%.4f  ", theta, atom.Totbonds, G(theta,atom));
			atom.Totbonds++;
			fprintf(outf, "G(%.1f,%.1f)=%.4f  ", theta, atom.Totbonds, G(theta,atom));
			atom.Totbonds++;
			fprintf(outf, "G(%.1f,%.1f)=%.4f\n", theta, atom.Totbonds, G(theta,atom));
			atom.Totbonds++;
		}
	}
	fprintf(outf, "hydrogen\n");
	atom.type = CARBON; 
	for (theta=0; theta<6.28; theta+=.1) {
		atom.Totbonds=0;
		while (atom.Totbonds<5) {
			fprintf(outf, "G(%.1f,%.1f)=%.4f  ", theta, atom.Totbonds, G(theta,atom));
			atom.Totbonds++;
			fprintf(outf, "G(%.1f,%.1f)=%.4f  ", theta, atom.Totbonds, G(theta,atom));
			atom.Totbonds++;
			fprintf(outf, "G(%.1f,%.1f)=%.4f\n", theta, atom.Totbonds, G(theta,atom));
			atom.Totbonds++;
		}
	}
	fclose(outf);
}

void plot_BCUINT()
{
	Float i, j;
	FILE *outf;

	outf = fopen("c:\\temp.ent", "w");
	fprintf(outf, "      KI,XX1,XX2\n");
	for (i=1; i<5; i++) {
		for (j=1; j<5; j++) {
			fprintf(outf, "BCUINT(hc, %.1f, %.1f) = %f			", i, j, BCUINT(2,i,j, NULL, NULL));
			fprintf(outf, "BCUINT(cc, %.1f, %.1f) = %f\n", i, j, BCUINT(1,i,j, NULL, NULL));
		}
	}
	fclose(outf);

}

/* Find angle between three atoms. */
static Float theta( BrenAtom i, BrenAtom j, BrenAtom k )
{
  Float angle;
  vector ij, ik;
  ij = minus(j.coord, i.coord);
  ik = minus(k.coord, i.coord);
  /* I don't think this is used.  I care slightly because I want to know if the
     real program uses any trig functions, since the Pentium III SIMD
     instructions can't do trig. */
  assert (0 && "Tim doesn't think we can get here.");
  angle = acos( dot(ij,ik)/(sqrt(dot(ij,ij))*sqrt(dot(ik,ik))) );
  return angle;
}

/* pSigmaPi ij */

Float pSigmaPi( BrenAtom i, BrenAtom j )
{
	Float term1, term2 = 0, result, expterm, R;
	int k,n;
	
	term1=0;
	
	/*check the neighbors of atom i, indexed from 0 to num_neighbors */
	for (n=0; n<i.num_neighbors; n++) {
		k = i.neighbor[n];
		if ((atm_num[k].number != i.number) && (atm_num[k].number != j.number)) {
			/*RHH=0.7415886997;  RCH=1.09; XDB[][][]; REG[][][]?*/
			if (i.type == HYDROGEN)
				R = 0.74144;
			else  /*it's carbon*/
				R = 1.1199;
			expterm = exp(4*((distween_atms(i,j)-R)-(distween_atms(i,atm_num[k])-R)));
			/* Next line is the only use of theta. */
			term1 = term1 + G(theta(i,j,atm_num[k]),i)*is_bonded(i,atm_num[k])*expterm;
		}
	}

	/*printf("term1 for atm %d and atm %d is: %f\n", i.number, j.number, term1);*/
	if (i.type == CARBON) {
		if (j.type==CARBON)
			term2 = BCUINT(1, i.Hbonds+1, i.Cbonds, NULL, NULL);
		if (j.type==HYDROGEN)
			term2 = BCUINT(2, i.Hbonds, i.Cbonds+1, NULL, NULL);
	}
	else
		term2 = 0;
	
	
	if (1+term1+term2 < 0) 
		printf("You're square rooting a negative number!\n");
	

	result = pow((1+term1+term2),-.5);
	
	/*printf("pSigmaPi for atm %d and atm %d is: %f\n", i.number, j.number, result);*/
	return result;
}

void plot_RADIC()
{
	int i, j;
	FILE *outf;

	outf = fopen("c:\\temp.ent", "w");
	fprintf(outf, "      KIKJ,XNT1,XNT2,CONJUG\n");
	for (i=1; i<5; i++) {
		for (j=1; j<5; j++) {
			fprintf(outf, "RAD(hh,%d,%d,1)=%.4f	", i, j, RADIC(2,2,(Float)i,(Float)j,1, NULL, NULL, NULL));
			fprintf(outf, "RAD(hc,%d,%d,1)=%.4f	", i, j, RADIC(1,2,(Float)i,(Float)j,1, NULL, NULL, NULL));
			fprintf(outf, "RAD(cc,%d,%d,1)=%.4f\n", i, j, RADIC(1,1,(Float)i,(Float)j,1, NULL, NULL, NULL));
		}
	}
	fprintf(outf, "      KIKJ,XNT1,XNT2,CONJUG\n");
	for (i=1; i<5; i++) {
		for (j=1; j<5; j++) {
			fprintf(outf, "RAD(hh,%d,%d,2)=%.4f	", i, j, RADIC(2,2,(Float)i,(Float)j,2, NULL, NULL, NULL));
			fprintf(outf, "RAD(hc,%d,%d,2)=%.4f	", i, j, RADIC(1,2,(Float)i,(Float)j,2, NULL, NULL, NULL));
			fprintf(outf, "RAD(cc,%d,%d,2)=%.4f\n", i, j, RADIC(1,1,(Float)i,(Float)j,2, NULL, NULL, NULL));
		}
	}
	fprintf(outf, "      KIKJ,XNT1,XNT2,CONJUG\n");
	for (i=1; i<5; i++) {
		for (j=1; j<5; j++) {
			fprintf(outf, "RAD(hh,%d,%d,3)=%.4f	", i, j, RADIC(2,2,(Float)i,(Float)j,3,NULL, NULL, NULL));
			fprintf(outf, "RAD(hc,%d,%d,3)=%.4f	", i, j, RADIC(1,2,(Float)i,(Float)j,3,NULL, NULL, NULL));
			fprintf(outf, "RAD(cc,%d,%d,3)=%.4f\n", i, j, RADIC(1,1,(Float)i,(Float)j,3, NULL, NULL, NULL));
		}
	}
	fclose(outf);

}

void plot_TOR()
{
	Float i, j, k;
	FILE *outf;

	outf = fopen("c:\\temp.ent", "w");
	fprintf(outf, "   XNT1, XNT2, CONJUG\n");
	for (i=1; i<5; i++) {
		for (j=1; j<5; j++) {
			for (k=1; k<5; k++) {
				fprintf(outf, "TOR(%.1f,%.1f,%.1f)=%.4f\n", i, j, k, TOR(i,j,k,NULL, NULL, NULL));
			} 
		}
	}
	fclose(outf);
}

Float dihedral(BrenAtom i, BrenAtom j, Float ijConj)
{
	int k,l,m,n;
	Float sum, omega, result;
	vector Rji, Rik, Rij, Rjl, ejik, eijl;
	
	sum=0;
	for (n=0; n<i.num_neighbors; n++) {
		k = i.neighbor[n];
		for (m=0; m<j.num_neighbors; m++) {
			l=j.neighbor[m];
			if (k!=i.number && k!=j.number &&
				l!=i.number && l!=j.number) {
				
				Rji = plus(j.coord,product(-1,i.coord));
				Rik = plus(i.coord,product(-1,atm_num[k].coord));
				Rij = product(-1,Rji);
				Rjl = plus(j.coord,product(-1,atm_num[l].coord));
				ejik = cross(Rji,Rik);
				ejik = product(1/dot(ejik,ejik),ejik);
				eijl = cross(Rij,Rjl);
				eijl = product(1/dot(eijl,eijl),eijl);
				omega = dot(ejik,eijl);
				sum += (1-cos(omega)*cos(omega))*is_bonded(i,atm_num[k])*is_bonded(j,atm_num[k]);
			}
		}
	}

	result = TOR(i.Totbonds,j.Totbonds,ijConj, NULL, NULL, NULL)*sum;
	/*printf("ijConj= %f\n",ijConj); 
	printf("TOR = %f\n", result/sum);
	printf("sum %d %d = %f\n", i.number, j.number, sum);
	printf("dihedral %d %d = %f\n", i.number, j.number, result); */
	return result;
}

/* bij, bond order modulated by neighbors of i and j*/

Float bond_order( BrenAtom i, BrenAtom j )
{
	int neighbor = -1,n;
	Float bondOrder, ijConj;

	/*search through neighbors of i to find j index wrt i*/
	for (n=0; n<i.num_neighbors; n++) {
		if (i.neighbor[n]==j.number) {
			neighbor = n;
			n=i.num_neighbors;/*break*/
		}
	}
	ijConj = i.Conj[neighbor];
	if (i.type==CARBON && j.type==CARBON) 
      bondOrder = (pSigmaPi(i,j)+pSigmaPi(j,i))/2 +
        RADIC(1,1, i.Totbonds, j.Totbonds, ijConj, NULL, NULL, NULL) +
        dihedral(i, j, ijConj); 
	else if (i.type==HYDROGEN && j.type==HYDROGEN)
      bondOrder = (pSigmaPi(i,j)+pSigmaPi(j,i))/2 +
        RADIC(2,2, i.Totbonds, j.Totbonds, 1, NULL, NULL, NULL);
	else	/*hc or ch*/
      bondOrder = (pSigmaPi(i,j)+pSigmaPi(j,i))/2 +
        RADIC(1,2, i.Totbonds, j.Totbonds, 1, NULL, NULL, NULL) +
        dihedral(i, j, ijConj); 
	return bondOrder;
}

/* Total energy for atom a. Ei = sumi-tot(VR-b*VA) */

Float energy( BrenAtom *pAtm )
{
	int j,n;
	int count = 0;
	Float en;
	en = 0;
	/*check the neighbors of atom i, indexed from 0 to num_neighbors */
	for (n=0; n<pAtm->num_neighbors; n++) {
		j = pAtm->neighbor[n];
		if (atm_num[j].number != pAtm->number) {
		  en = en + repulsive_e(*pAtm,atm_num[j]) - 
		    bond_order(*pAtm,atm_num[j])*attractive_e(*pAtm,atm_num[j]);
		  ++count;
		}
	}
	pAtm->energy = en;
	if(count && pAtm->number < 3)
	  printf("the energy of atom %3d is %8.4e; %2d neighbors\n", pAtm->number, en, count);
	return en;
}


/* Read in two atoms and find the attractive energy of the first contributed by 
   the second. VA(rij) */

Float attractive_e( BrenAtom x, BrenAtom y )		
{
	Float energy, distance, B1 = 0, B2 = 0, B3 = 0;
	Float beta1 = 0, beta2 = 1, beta3 = 1;
	 
	if ((x.type == HYDROGEN) && (y.type == HYDROGEN)) {
		B1 = 29.63259;
		B2 = 0;
		B3 = 0;
		beta1 = -1.71589;
		beta2 = 1;
		beta3 = 1;
	}
	else if ((x.type == CARBON) && (y.type == CARBON)) {
		B1 = 12388.79198;
		B2 = 17.56741;
		B3 = 30.71493;
		beta1 = -4.72045;
		beta2 = -1.43321;
		beta3 = -1.38269;
	}
	else if (((x.type == HYDROGEN) && (y.type == CARBON)) ||
	       ((x.type == CARBON) && (y.type == HYDROGEN)))  {			
		B1 = 32.35519;
		B2 = 0;
		B3 = 0;
		beta1 = -1.43446;
		beta2 = 1;
		beta3 = 1;
	}
	else
		printf("attractive_e: not c or h");

	distance = distween_atms( x, y );
	
	energy = is_bonded(x, y)*(B1*exp(beta1*distance)+B2*exp(beta2*distance)+B3*exp(beta3*distance));
#ifndef NDEBUG
	if(0) printf("attract e : %d and %d is %e is_bonded %4.2f distance %4.2f\n",
	       x.number, y.number, energy, is_bonded(x, y), distance);
#endif
	return energy;
}

/* Read in two atoms and find the repulsive energy of the first contributed by 
   the second. VR(rij) */

Float repulsive_e( BrenAtom x, BrenAtom y )		
{
	Float energy, distance, Q = 0, A = 0, alpha = 0;

	if ((x.type == HYDROGEN) && (y.type == HYDROGEN)) {
		Q = 0.37047;
		A = 32.81736;
		alpha = -3.53630;
	}
	else if ((x.type == CARBON) && (y.type == CARBON)) {
		Q = 0.31346;
		A = 10953.54416;
		alpha = -4.74654;
	}
	else if (((x.type == HYDROGEN) && (y.type == CARBON)) ||
	       ((x.type == CARBON) && (y.type == HYDROGEN)))  {			
		Q = 0.34078;
		A = 149.94099;
		alpha = -4.10255;
	}
	else 
		printf("repulsive_e: not c or h");
	 
	distance = distween_atms( x, y );
	energy = is_bonded(x, y)*(1+Q/distance)*A*exp(alpha*distance);
	 /* printf("att e : %d and %d is %f\n", x.number, y.number, energy);*/
	return energy;
}

/* Read in two atoms, return Float between 0 and 1, one being bonded and 
   0 unbonded. fij(r) */

Float is_bonded( BrenAtom x, BrenAtom y)
{
	Float R1 = 0, R2 = 0;
	Float distance, bondingness;
	distance = distween_atms( x, y );
/*	printf("The distance between atom %d and %d is %lf\n", 
		x.number, y.number, distance);*/
	
	if ((x.type == HYDROGEN) && (y.type == HYDROGEN)) {           
		R1 = 1.1;
		R2 = 1.7;       
	}
	else if ((x.type == CARBON) && (y.type == CARBON)) {     
		R1 = 1.7;
		R2 = 2;
	}
	else if (((x.type == HYDROGEN) && (y.type == CARBON)) ||     
	       ((x.type == CARBON) && (y.type == HYDROGEN)))  {			
		R1 = 1.3;
		R2 = 1.8;
	}
	else {
		static int cnt_msg = 0;
		if(!cnt_msg++) fprintf(stderr, "you didn't input c or h !\n");
		return 0;
	}
    
	if ( distance < R1 )
		return 1;
	else if ( distance > R2 )
		return 0;
	else {
		bondingness = (1 + cos( M_PI*(distance - R1)/(R2-R1) ) )/2;
		return bondingness; 
	}
}

/* Read in two atoms, return the distance between them. rij*/

Float distween_atms( BrenAtom i, BrenAtom j)
{
	Float xdis, ydis, zdis, tot_distance;
	
	xdis = (i.coord.x - j.coord.x);
	ydis = i.coord.y - j.coord.y;
	zdis = i.coord.z - j.coord.z;
	tot_distance = sqrt( xdis*xdis + ydis*ydis + zdis*zdis );
#ifndef NDEBUG
	if(0)
	  printf("dist %2d %2d  %7.2f %5.1f %5.1f %5.1f xs %7.2f %7.2f\n",
		 i.number, j.number, xdis, ydis, zdis, tot_distance, 
		 i.coord.x, j.coord.x);
#endif
	return tot_distance;
}

/* The following code was found unused in bren2.c.  It updates the fields
  Float Conj[MAXNBORS]; 
  Float Hbonds, Cbonds, Totbonds;

  in the BrenAtom structure defined in brenner.h, but nobody referenced those
  fields.  Tim Freeman 30 Jul 2000 */

void find_bonds(BrennerMainInfo *info);
static Float conj2( BrenAtom *atm_num, int i, int j );
static Float conj1( BrenAtom i, BrenAtom k );
static void find_conjugation(BrenAtom *atm_num, int num_atms);

/* Find F(xik) to see if conjugated. */
/* Only called from conj2. */
static Float conj1( BrenAtom i, BrenAtom k )
{
	Float x, F;
	/* messed up by different totbonds for each pair*/
	x = k.Totbonds - is_bonded(i,k);
	if (x <= 2)
		return 1;
	else if (x >= 3)
		return 0;
	else {
		F=(1+cos(M_PI*(x-2)))/2;
		return F;
	}
}

/* Find Nijconj to see if conjugated.*/
/* Only called from find_conjugation. */
static Float conj2( BrenAtom *atm_num, int i, int j )
{
	int k;
	Float term1, term2, N;

	term1=0;
	term2=0;
	/*check the neighbors of atom i, indexed from 0 to num_neighbors*/
	for (k=0; k < atm_num[i].num_neighbors; k++) {
		int kk = atm_num[i].neighbor[k];
		if ((kk != i) && (kk != j) && (atm_num[kk].type == CARBON))
			term1 += is_bonded(atm_num[i], atm_num[kk])
			  * conj1(atm_num[i], atm_num[kk]);
	}
	for (k=0; k < atm_num[j].num_neighbors; k++) {
		int kk = atm_num[j].neighbor[k];
		if ((kk != i) && (kk != j) && (atm_num[kk].type == CARBON))
			term2 += is_bonded(atm_num[j], atm_num[kk])
			  * conj1(atm_num[j], atm_num[kk]);
	}
	N = 1 + term1*term1 + term2*term2;
	/*printf("Nij conj between atom %d and atom %d is: %.3f \n", i, j, N);*/	
	return N;
}

/* The only consequence of this is to update the Conj field of the given
   BrenAtom's, but we never reference that field. */
/* FIXME Move this to junk. */
static void find_conjugation(BrenAtom *atm_num, int num_atms)
{
	int i, j, n;
	for (i=0; i<num_atms; i++) {
		/*check the neighbors of atom i, indexed from 0 to num_neighbors */
		for (n=0; n<atm_num[i].num_neighbors; n++) {
			j = atm_num[i].neighbor[n];
			atm_num[i].Conj[n] = 1;
			if (atm_num[i].type==CARBON && atm_num[j].type==CARBON) /* only conjugation for cc*/
				atm_num[i].Conj[n] = conj2( atm_num, i, j);
		}
	}
}

/* Take an atom and add up all the bondingnesses to that atom, return to atom
   struct.  Repeat for all the atoms. NiT = NiC + NiH */

void find_bonds(BrennerMainInfo *info)			
{
	int i, j, n;
	Float x;
	int num_atms = info->num_atms;
	BrenAtom *atm_num = info->atm_num;
	for (i=0; i<num_atms; i++) {
		/*check the neighbors of atom i, indexed from 0 to num_neighbors */
		atm_num[i].Cbonds = 0;
		atm_num[i].Hbonds = 0;
		for (n=0; n<atm_num[i].num_neighbors; n++) {
			j = atm_num[i].neighbor[n];
			x = is_bonded( atm_num[i], atm_num[j] );
			/*printf("The bondingness between atoms %d and %d is %lf\n", i, j, x);*/
			if (atm_num[j].type == CARBON) {
				atm_num[i].Cbonds = atm_num[i].Cbonds + x;
			}
			else if (atm_num[j].type == HYDROGEN) {
				atm_num[i].Hbonds = atm_num[i].Hbonds + x;
			}
			else if (atm_num[j].type == SILICON) {
				;
			}
			else
			{
			  fprintf(stderr,"You didn't input C, or H! atom %d type %d\n",
				  j, atm_num[j].type);
			  my_exit(-1);
			}
		}
        /* This is the only place we assign to atm_num[...].Totbonds. */
		atm_num[i].Totbonds = atm_num[i].Cbonds + atm_num[i].Hbonds;
#ifndef NDEBUG
		if(0) printf("Atom %d has a total of %f bonds\n",
                     i, atm_num[i].Totbonds);
#endif
	}
}

/* G and findQ were discovered to be unused in bren2.c.  Tim Freeman 30 Jul
   2000 */

static Float G( Float angle, BrenAtom i);
static Float findQ(Float input);
/*Q modulates between angular splines for under and over-coordinated carbon*/

Float findQ(Float input)
{
	Float result;

	if (input<3.2)
		return 1;
	else if (input>3.7)
		return 0;
	else {
		result = (1+cos(2*M_PI*(input-3.2)))/2;
		return result;
	}
}

/* G(theta ijk) from Gh, Gc and Gammac splines*/

Float G( Float angle, BrenAtom i)
{
	int JJ, IG, IG1;
	Float COSTH, GANGLE, GANGLE1;

	COSTH=cos(angle);
	if (COSTH > 1) COSTH = 1;
	if (COSTH < -1) COSTH = -1;
	
	if (i.type==CARBON) {
		IG=IGC[(int)floor(-COSTH*12)+13];
		if (IG!=4) { 
			GANGLE = SPGC[IG][1]+SPGC[IG][2]*COSTH;
			for (JJ=3; JJ<7; JJ++) 
				GANGLE += SPGC[IG][JJ]*pow(COSTH,JJ-1);
		}
		else {
			GANGLE = SPGC[1][IG]+SPGC[2][IG]*COSTH;
			IG1=IG+1;
			GANGLE1 = SPGC[1][IG1]+SPGC[2][IG1]*COSTH;
			for (JJ=3; JJ<7; JJ++) {
				GANGLE += SPGC[JJ][IG]*pow(COSTH, JJ-1);
				GANGLE1 += SPGC[JJ][IG1]*pow(COSTH, JJ-1);
			}
			GANGLE += findQ(i.Totbonds)*(GANGLE1-GANGLE);
		}
	}
	else {
		IG=IGH[(int)floor(-COSTH*12)+13];
		GANGLE=SPGH[1][IG]+SPGH[2][IG]*COSTH;
		for (JJ=3; JJ<7; JJ++) {
			GANGLE += SPGH[JJ][IG]*pow(COSTH,JJ-1);
		}
	}
	/*printf("(int)floor(-COSTH*12)+13 = %f\n", (int)floor(-COSTH*12)+13);
	printf("GANGLE = %f\n", GANGLE);*/
	return GANGLE;
}

